<?php
/**
 * Template Parser
 * @package     Template
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Template_Parser
{
    /**
     * 配信方式
     * @var string static / dynamic
     */
    public $output_method;
    /**
     * ページDAOオブジェクト
     * @var Page_Dao
     */
    private $page_dao;
    /**
     * テンプレートDAOオブジェクト
     * @var Template_Dao
     */
    private $template_dao;


    /**
     * コンストラクタ
     */
    public function __construct()
    {
        $this->page_dao = new Page_Dao();
        $this->template_dao = new Template_Dao();
    }

    /**
     * パース処理
     * @param string $source_code
     * @param string $page_path
     * @return Metag_Template $metag_template
     */
    public function parse($source_code, $page_path = null)
    {
        $metag_template = new Metag_Template();

        // Meタグとhtml部分に分けて抽出
        $metag_list = preg_split('/(<!--{.*?}-->)/', $source_code, null, PREG_SPLIT_DELIM_CAPTURE);

        $size = count($metag_list);
        $current = array($metag_template);
        $is_inside_repeat = false;
        $repeat_id = '';
        $ref_repeat_id = '';
        $ref_page_id = '';
        $ref_template_id = '';
        $ref_page_list = array();
        $ref_template_list = array();
        $module_view_list = array();
        for ($i = 0; $i < $size; $i++) {
            if ($i % 2 === 0) {
                // 偶数はhtml部分
                $metag_html = $this->parse_html($metag_list[$i]);
                $current[count($current) - 1]->child_nodes[] = $metag_html;
            } else {
                // 奇数はMeタグ部分
                $metag = $this->parse_metag($metag_list[$i]);
                $metag->page_path = $page_path;
                if ($metag->is_open_tag) {
                    // 開始タグ
                    $current[count($current) - 1]->child_nodes[] = $metag;
                    $current[] = $metag;
                    // 要素配列に追加
                    $metag_template->id_node_list[$metag->id] = $metag;
                    if (strlen($metag->ref_page_id) > 0) {
                        $ref_page_list[] = $metag->ref_page_id;
                    }
                    if (strlen($metag->ref_template_id) > 0) {
                        $ref_template_list_temp = array_map('trim', explode(',', $metag->ref_template_id));
                        $ref_template_list = array_merge($ref_template_list, $ref_template_list_temp);
                        $metag_template->ref_template_repeat[] = $metag;
                    }
                    if ($metag->node_type === 'module' && !empty($metag->module_name)) {
                        $module_view_list[] = $metag->module_name;
                    }
                } else {
                    // 終了タグ
                    array_pop($current);
                }
                // repeat制御
                if ($metag->node_type === 'repeat') {
                    if ($metag->is_open_tag) {
                        $is_inside_repeat = true;
                        $repeat_id = $metag->id;
                        $ref_repeat_id = (strlen($metag->ref_id) > 0) ? $metag->ref_id : '';
                        $ref_page_id = (strlen($metag->ref_page_id) > 0) ? $metag->ref_page_id : '';
                        $ref_template_id = (strlen($metag->ref_template_id) > 0) ? $metag->ref_template_id : '';
                    } else {
                        $is_inside_repeat = false;
                    }
                } else {
                    if ($is_inside_repeat) {
                        $metag->repeat_id = $repeat_id;
                        $metag->ref_repeat_id = $ref_repeat_id;
                        $metag->ref_page_id = $ref_page_id;
                        $metag->ref_template_id = $ref_template_id;
                    }
                }
            }
        }
        $metag_template->ref_page_list = array_unique($ref_page_list);
        $metag_template->ref_template_list = array_unique($ref_template_list);
        $metag_template->module_view_list = array_unique($module_view_list);
        return $metag_template;
    }

    /**
     * Meタグをパースして、タグオブジェクトを生成する
     * @param string $node_value
     * @return object $tag_object
     */
    private function parse_metag($node_value)
    {
        $tag_text = $node_value;
        
        // タグ前後の区切りを削除
        $node_value = preg_replace('/^<!--{|}-->$/', '', $node_value);
        // 開始・終了タグを判定
        $first_char = substr($node_value, 0, 1);
        // 開始タグ
        $is_open_tag = true;
        if ($first_char === '/') {
            // 終了タグ
            $is_open_tag = false;
        }
        // /を削除
        $node_value = trim($node_value, '/');

        // 最初の空白文字まで分割する
        list($tag_name, $tag_attr) = explode(' ', $node_value, 2);
        
        // タグ名確認
        if (!Metag_Logic::exist_type(strtolower($tag_name))) {
            throw new Me_Exception(611, array($tag_name, $tag_text));
        }

        // class名作成
        $class_name = 'Metag_' . ucfirst($tag_name);

        // オブジェクトを作成して、属性をセット
        $tag_object = new $class_name();
        $tag_object->is_open_tag = $is_open_tag;

        // 終了タグは属性の取得はしない
        if (!$is_open_tag) {
            return $tag_object;
        }

        // 属性を取得
        preg_match_all('/[[:alnum:]_]+=(""|".*?[^\\\\]"|\'\'|\'.*?[^\\\\]\')/', $tag_attr, $matches);
        foreach ($matches[0] as $match) {
            list($attr_name, $attr_value) = explode('=', $match, 2);
            $attr_value = substr($attr_value, 1, strlen($attr_value) - 2);
            if (property_exists($tag_object, $attr_name)) {
                $tag_object->$attr_name = $attr_value;
            }
        }

        // ref_page_idにパスが指定されていた場合は、IDに変換する。
        if (strlen($tag_object->ref_page_id) && !is_numeric($tag_object->ref_page_id)) {
            $tag_object->ref_page_id = $this->page_dao->select_id_by_page_path($tag_object->ref_page_id);
        }
        // ref_template_idにパスが指定されていた場合は、IDに変換する。
        if (strlen($tag_object->ref_template_id) && !is_numeric($tag_object->ref_template_id)) {
            // カンマで分割
            $ref_template_list_temp = array_map('trim', explode(',', $tag_object->ref_template_id));
            foreach ($ref_template_list_temp as $ref_template_key => $ref_template_item) {
                // numericでなければ、検索してIDを取得
                if (!is_numeric($ref_template_item)) {
                    $ref_template_list_temp[$ref_template_key] = $this->template_dao->select_id_by_template_path($ref_template_item);
                }
            }
            // カンマ区切りにして格納
            $tag_object->ref_template_id = implode(',', $ref_template_list_temp);
        }

        // 配信方式をセット
        $tag_object->output_method = $this->output_method;

        return $tag_object;
    }

    /**
     * HTMLをパースして、Metag_Htmlオブジェクトを生成する
     * @param string $node_value
     * @return Metag_Html $metag_html
     */
    private function parse_html($node_value)
    {
        $metag_html = new Metag_Html();
        $metag_html->node_value = $node_value;
        $metag_html->child_nodes = array();

        // imgタグの属性を抽出
        if (preg_match('/<img.*?>/i', $node_value, $matches_img)) {
            // 属性を取得
            $metag_html->img_attr = array();
            preg_match_all('/[[:alnum:]_]+=(""|".*?[^\\\\]"|\'\'|\'.*?[^\\\\]\')/', $matches_img[0], $matches_img_attr);
            foreach ($matches_img_attr[0] as $match) {
                list($attr_name, $attr_value) = explode('=', $match, 2);
                $attr_value = substr($attr_value, 1, strlen($attr_value) - 2);
                $metag_html->img_attr[$attr_name] = $attr_value;
            }
        }
        // aタグの属性を抽出
        if (preg_match('/<a.*?>/i', $node_value, $matches_a)) {
            // 属性を取得
            $metag_html->a_attr = array();
            preg_match_all('/[[:alnum:]_]+=(""|".*?[^\\\\]"|\'\'|\'.*?[^\\\\]\')/', $matches_a[0], $matches_a_attr);
            foreach ($matches_a_attr[0] as $match) {
                list($attr_name, $attr_value) = explode('=', $match, 2);
                $attr_value = substr($attr_value, 1, strlen($attr_value) - 2);
                $metag_html->a_attr[$attr_name] = $attr_value;
            }
        }
        return $metag_html;
    }
}